@@ -2,7 +2,7 @@
 #
 # vulture - Find dead code.
 #
-# Copyright (c) 2012-2019 Jendrik Seipp (jendrikseipp@gmail.com)
+# Copyright (c) 2012-2020 Jendrik Seipp (jendrikseipp@gmail.com)
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the
@@ -35,9 +35,10 @@ import string
 import sys
 
 from vulture import lines
+from vulture import noqa
 from vulture import utils
 
-__version__ = "1.4"
+__version__ = "1.6"
 
 DEFAULT_CONFIDENCE = 60
 
@@ -46,6 +47,17 @@ IGNORED_VARIABLE_NAMES = {"object", "self"}
 if sys.version_info < (3, 4):
     IGNORED_VARIABLE_NAMES |= {"True", "False"}
 
+ERROR_CODES = {
+    "attribute": "V101",
+    "class": "V102",
+    "function": "V103",
+    "import": "V104",
+    "method": "V105",
+    "property": "V106",
+    "variable": "V107",
+    "unreachable_code": "V201",
+}
+
 
 def _get_unused_items(defined_items, used_names):
     unused_items = [
@@ -84,8 +96,12 @@ def _ignore_import(filename, import_name):
 
 
 def _ignore_function(filename, function_name):
-    return _is_special_name(function_name) or (
-        function_name.startswith("test_") and _is_test_file(filename)
+    return function_name.startswith("test_") and _is_test_file(filename)
+
+
+def _ignore_method(filename, method_name):
+    return _is_special_name(method_name) or (
+        method_name.startswith("test_") and _is_test_file(filename)
     )
 
 
@@ -160,7 +176,9 @@ class Item(object):
                 self.message, filename, self.first_lineno
             )
         else:
-            prefix = "_." if self.typ in ["attribute", "property"] else ""
+            prefix = ""
+            if self.typ in ["attribute", "method", "property"]:
+                prefix = "_."
             return "{}{}  # unused {} ({}:{:d})".format(
                 prefix, self.name, self.typ, filename, self.first_lineno
             )
@@ -196,6 +214,7 @@ class Vulture(ast.NodeVisitor):
         self.defined_classes = get_list("class")
         self.defined_funcs = get_list("function")
         self.defined_imports = get_list("import")
+        self.defined_methods = get_list("method")
         self.defined_props = get_list("property")
         self.defined_vars = get_list("variable")
         self.unreachable_code = get_list("unreachable_code")
@@ -213,6 +232,7 @@ class Vulture(ast.NodeVisitor):
     def scan(self, code, filename=""):
         code = utils.sanitize_code(code)
         self.code = code.splitlines()
+        self.noqa_lines = noqa.parse_noqa(self.code)
         self.filename = filename
         try:
             node = ast.parse(code, filename=self.filename)
@@ -300,6 +320,7 @@ class Vulture(ast.NodeVisitor):
             + self.unused_classes
             + self.unused_funcs
             + self.unused_imports
+            + self.unused_methods
             + self.unused_props
             + self.unused_vars
             + self.unreachable_code
@@ -348,6 +369,10 @@ class Vulture(ast.NodeVisitor):
             self.defined_imports, self.used_names | self.used_attrs
         )
 
+    @property
+    def unused_methods(self):
+        return _get_unused_items(self.defined_methods, self.used_attrs)
+
     @property
     def unused_props(self):
         return _get_unused_items(self.defined_props, self.used_attrs)
@@ -393,13 +418,23 @@ class Vulture(ast.NodeVisitor):
                 self.unreachable_code,
                 name,
                 node,
-                last_node=node.body[-1],
+                last_node=node.body
+                if isinstance(node, ast.IfExp)
+                else node.body[-1],
                 message="unsatisfiable '{name}' condition".format(**locals()),
                 confidence=100,
             )
         elif utils.condition_is_always_true(node.test):
             else_body = node.orelse
-            if else_body:
+            if name == "ternary":
+                self._define(
+                    self.unreachable_code,
+                    name,
+                    else_body,
+                    message="unreachable 'else' expression",
+                    confidence=100,
+                )
+            elif else_body:
                 self._define(
                     self.unreachable_code,
                     "else",
@@ -461,14 +496,20 @@ class Vulture(ast.NodeVisitor):
         confidence=DEFAULT_CONFIDENCE,
         ignore=None,
     ):
+        def ignored(lineno):
+            return (
+                (ignore and ignore(self.filename, name))
+                or _match(name, self.ignore_names)
+                or noqa.ignore_line(self.noqa_lines, lineno, ERROR_CODES[typ])
+            )
+
         last_node = last_node or first_node
         typ = collection.typ
-        if (ignore and ignore(self.filename, name)) or _match(
-            name, self.ignore_names
-        ):
+        first_lineno = lines.get_first_line_number(first_node)
+
+        if ignored(first_lineno):
             self._log('Ignoring {typ} "{name}"'.format(**locals()))
         else:
-            first_lineno = lines.get_first_line_number(first_node)
             last_lineno = lines.get_last_line_number(last_node)
             collection.append(
                 Item(
@@ -529,7 +570,26 @@ class Vulture(ast.NodeVisitor):
             utils.get_decorator_name(decorator)
             for decorator in node.decorator_list
         ]
-        typ = "property" if "@property" in decorator_names else "function"
+
+        first_arg = None
+        if node.args.args:
+            try:
+                first_arg = node.args.args[0].arg
+            except AttributeError:
+                # Python 2.7
+                first_arg = node.args.args[0].id
+
+        if "@property" in decorator_names:
+            typ = "property"
+        elif (
+            "@staticmethod" in decorator_names
+            or "@classmethod" in decorator_names
+            or first_arg == "self"
+        ):
+            typ = "method"
+        else:
+            typ = "function"
+
         if any(
             _match(name, self.ignore_decorators) for name in decorator_names
         ):
@@ -540,8 +600,11 @@ class Vulture(ast.NodeVisitor):
             )
         elif typ == "property":
             self._define(self.defined_props, node.name, node)
+        elif typ == "method":
+            self._define(
+                self.defined_methods, node.name, node, ignore=_ignore_method
+            )
         else:
-            # Function is not a property.
             self._define(
                 self.defined_funcs, node.name, node, ignore=_ignore_function
             )
@@ -556,6 +619,9 @@ class Vulture(ast.NodeVisitor):
     def visit_If(self, node):
         self._handle_conditional_node(node, "if")
 
+    def visit_IfExp(self, node):
+        self._handle_conditional_node(node, "ternary")
+
     def visit_Import(self, node):
         self._add_aliases(node)
 
